theory Basics
imports Main
begin

section \<open>Documentation panel\<close>

text \<open>
  \<bullet> Notable manuals: jedit, jedit-manual, isar-ref, main, prog-prove

  \<bullet> Examples:
    @{file "~~/src/HOL/ex/Seq.thy"}
    @{file "~~/src/HOL/ex/ML.thy"}
    @{file "~~/src/Tools/SML/Examples.thy"}
\<close>


section \<open>Text structure and navigation\<close>

text \<open>
  \<bullet> SideKick provides a tree view of various file formats.

  \<bullet> Navigation works via Back and Forward buttons, like in a web browser.

  \<bullet> Text folds can be manipulated systematically (e.g. to collapse sub-proofs).

  \<bullet> CONTROL-hover (on Linux/Windows) or COMMAND-hover (on Mac OS X)
    reveals additional information, notably hyper-links.

  For example:
    @{url "http://isabelle.in.tum.de"}
    @{file "~~/src/HOL/Library"}
    @{term "x + 1"}
\<close>


subsection \<open>Exercise\<close>

text \<open>
  Explore some example theory source and identify formal entities using the IDE.

  \<triangleright> Identify at least 5 different sub-languages of Isabelle.

  \<triangleright> Determine at least 5 different kinds of formal entities seen
    in example theories.

  \<triangleright> Connect use of formal entities with their definitions via navigation.

  \<triangleright> Find theorems and constants by explicit query on the context.
\<close>


section \<open>Isabelle symbols\<close>

text \<open>
  \<bullet> Isabelle supports infinitely many mathematical symbols: \<foobar>, \<baz>

  \<bullet> Finitely many symbols are rendered via Unicode: e.g. "bar" as "\<bar>"

  \<bullet> Some notable symbols with abbreviations for input (see also Symbols panel):

    symbol    name with backslash    abbreviation
    \<lambda>         \lambda                %
    \<Rightarrow>        \Rightarrow            =>
    \<Longrightarrow>       \Longrightarrow        ==>
    \<And>         \And                   !!
    \<equiv>         \equiv                 ==
    \<forall>         \forall                !
    \<exists>         \exists                ?
    \<longrightarrow>       \longrightarrow        -->
    \<and>         \and                   &
    \<or>         \or                    |
    \<not>         \not                   ~
    \<noteq>         \noteq                 ~=
    \<in>         \in                    :
    \<notin>         \notin                 ~:
\<close>


subsection \<open>Exercise\<close>

text \<open>
  \<triangleright> Write some logical propositions using Isabelle symbols.

  \<triangleright> Write propositions that are nested inside a different sub-language of Isabelle.

  For example:
\<close>

prop "A \<and> B \<longrightarrow> B \<and> A"
prop "\<not> x = y"
prop "x \<noteq> y"
prop "x \<le> y"
prop "\<forall>x y. x \<le> y"
prop "\<exists>x y. x \<le> y"


section \<open>Logical language: Isabelle/HOL\<close>

text \<open>
  Main kinds of formal entities:

  \<bullet> types: simple
  \<bullet> terms: \<lambda>-calculus with \<alpha>\<beta>\<eta>-equality
  \<bullet> propositions: terms of type bool

  Theorems are proven propositions; facts always occur as lists of theorems.
\<close>

typ nat
typ bool
typ "'a \<times> 'b"
typ "'a \<Rightarrow> 'b"

term "x :: nat"
term Suc
term "\<lambda>y. x + Suc y"

prop "x = y"
prop "A \<longrightarrow> B"
prop "\<forall>x. A x \<longrightarrow> B x"

thm refl sym trans
thm conjI conjunct1 conjunct2
thm impI mp
thm allI spec exI exE


subsection \<open>Exercise\<close>

text \<open>
  \<triangleright> Write some simple logical statements, and prove or disprove them using
    suitable Isabelle tools,
    e.g. automated proof methods, Sledgehammer, quickcheck, nitpick.

  For example:
\<close>

lemma "(\<exists>x. \<forall>y. P x y) \<longrightarrow> (\<forall>y. \<exists>x. P x y)" by auto
lemma "(\<forall>y. \<exists>x. P x y) \<longrightarrow> (\<exists>x. \<forall>y. P x y)" oops -- \<open>Quickcheck found a counterexample\<close>


section \<open>Rule statements: Isabelle/Pure\<close>

text \<open>
  \<bullet> Isabelle/Pure provides minimal logic connectives, to express
    Natural Deduction rule statements.
\<close>

prop "A \<Longrightarrow> B"      -- \<open>Pure implication: logical entailment\<close>
prop "\<And>x::'a. B x"  -- \<open>Pure quantification: local eigen-variables\<close>

prop "A\<^sub>1 \<Longrightarrow> A\<^sub>2 \<Longrightarrow> A\<^sub>3 \<Longrightarrow> C"   -- \<open>Horn clause\<close>
prop "(\<And>x. A x \<Longrightarrow> B x) \<Longrightarrow> C"  -- \<open>nested Horn clause: sub-proof with local context\<close>


text \<open>
  \<bullet> Isabelle/Isar provides separate notation for structured rule statements.
\<close>

thm impI mp
thm allI spec exI exI

print_statement impI mp
print_statement allI spec exI exI

end