section \<open>Insertion Sort and Multisets\<close>

theory Insertion_Sort_Multiset_Solution
imports Main "~~/src/HOL/Library/Multiset"
begin

subsection \<open>Definition of insertion sort as before\<close>

fun ins :: "'a::ord \<Rightarrow> 'a list \<Rightarrow> 'a list"
where
  "ins x [] = [x]"
| "ins x (y # ys) = (if x \<le> y then x # y # ys else y # ins x ys)"

definition isort :: "'a::ord list \<Rightarrow> 'a list"
  where "isort xs = foldr ins xs []"


subsection \<open>Exercise: characterization of permuted result via multisets\<close>

text \<open>
  \<triangleright> Use the concept of multisets to express the desired property of
    permutation of results.

  Hint: This requires import of "~~/src/HOL/Library/Multiset" from
  the Supplemental Isabelle/HOL Library.

  \<triangleright> Prove the main theorem by re-using lemmas from the old version
    without multisets.

  Hint: The 'abbreviation' command helps to adjust abstract syntax,
  without any logical impact.
\<close>


subsection \<open>Solution\<close>

text \<open>
  Subsequently we port the old proof without multisets, by adjusting the
  syntax and lemmas a bit.
\<close>

experiment
begin

abbreviation count_multiset :: "'a \<Rightarrow> 'a list \<Rightarrow> nat"  ("\<sharp>_'(_')")
  where "count_multiset x ys \<equiv> count (multiset_of ys) x"

lemma count_multiset_ins:
  "\<sharp>x(ins y ys) = (if x = y then Suc (\<sharp>x(ys)) else \<sharp>x(ys))"
  by (induct ys) auto

lemma count_multiset_isort: "\<sharp>x(isort ys) = \<sharp>x(ys)"
  by (induct ys) (auto simp: isort_def count_multiset_ins)

theorem multiset_isort: "multiset_of (isort xs) = multiset_of xs"
  by (simp add: multiset_eq_iff count_multiset_isort)

end

text \<open>
  The following proof (by Stefan Berghofer) uses multiset
  properties more directly.
\<close>

lemma multiset_ins: "multiset_of (ins x xs) = multiset_of xs + {#x#}"
  by (induct xs) (auto simp: add_ac)

theorem multiset_isort: "multiset_of (isort xs) = multiset_of xs"
  by (induct xs) (auto simp: isort_def multiset_ins)

end
