section \<open>Multiset as quotient type over list\<close>

theory Multiset_Quotient_Solution
imports Main
begin

subsection \<open>Auxiliary notation\<close>

text \<open>
  Readable notation for lifting combinators,
  see @{file "~~/src/HOL/Transfer.thy"}
\<close>

notation rel_fun (infixr "\<^bold>\<Longrightarrow>" 55)
notation map_fun (infixr "\<^bold>\<Rightarrow>" 55)


subsection \<open>Counting occurrences of elements\<close>

fun count :: "'a \<Rightarrow> 'a list \<Rightarrow> nat"  ("\<sharp>_'(_')")
where
  "\<sharp>x([]) = 0"
| "\<sharp>x(y#ys) = (if x = y then Suc (\<sharp>x(ys)) else \<sharp>x(ys))"

lemma count_append: "\<sharp>x(xs @ ys) = \<sharp>x(xs) + \<sharp>x(ys)"
  by (induct xs) auto


subsection \<open>Exercise: Quotient type definition\<close>

text \<open>
  \<triangleright> Define @{term "list_equiv :: 'a list \<Rightarrow> 'a list \<Rightarrow> bool"}
    to identify lists according the the multiplicity of
    each element, ignoring order of occurrence.

  \<triangleright> Define a type constructor @{text "'a mset"} as quotient of
    @{typ "'a list"}, using the command 'quotient_type'.
\<close>


subsection \<open>Solution\<close>

definition list_equiv :: "'a list \<Rightarrow> 'a list \<Rightarrow> bool"  (infix "\<approx>" 50)
  where "xs \<approx> ys \<longleftrightarrow> (\<forall>a. \<sharp>a(xs) = \<sharp>a(ys))"

quotient_type 'a mset = "'a list" / list_equiv
  morphisms list mset
  by (rule equivpI)
     (simp_all add: list_equiv_def reflp_def symp_def transp_def)


subsection \<open>Exercise: type class instantiation\<close>

text \<open>
  \<triangleright> Establish the following type class instantiation:

    instantiation mset :: (linorder) monoid_add
    begin
      ...
    end

  Hint: define zero_mset and plus_mset via 'lift_definition',
  and finish the 'instance' proof.
\<close>


subsection \<open>Solution\<close>

instantiation mset :: (linorder) monoid_add
begin

lift_definition zero_mset :: "'a mset" is "[]" .
lift_definition plus_mset :: "'a mset \<Rightarrow> 'a mset \<Rightarrow> 'a mset"
  is "\<lambda>xs ys. xs @ ys"
  by (simp add: list_equiv_def count_append)

instance
proof
  fix a b c :: "'a mset"
  show "(a + b) + c = a + (b + c)"
    by (induct a, induct b, induct c) (simp add: plus_mset.abs_eq)
  show "0 + a = a"
    by (induct a) (simp add: zero_mset.abs_eq plus_mset.abs_eq)
  show "a + 0 = a"
    by (induct a) (simp add: zero_mset.abs_eq plus_mset.abs_eq)
qed

end

end
